@@ -9,14 +9,9 @@ class Token(ABC):
 
     @abstractmethod
     def authenticate(self, url_connection):
-
-        try:
-            from zcrmsdk.src.com.zoho.crm.api.util import APIHTTPConnector
-        except Exception:
-            from ...crm.api.util import APIHTTPConnector
-
+        
         """
-        This method to set token to APIHTTPConnector instance
+        The method to set token to APIHTTPConnector instance
 
         Parameters:
             url_connection (APIHTTPConnector) : A APIHTTPConnector class instance.
