@@ -36,3 +36,11 @@ from . import blue_print
 from . import file
 from . import notification
 from . import query
+
+from . import inventory_templates
+from . import email_templates
+from . import field_attachments
+from . import send_mail
+from . import assignment_rules
+from . import pipeline
+from . import wizards
\ No newline at end of file
