@@ -30,7 +30,7 @@ class Currency(object):
 		The method to get the symbol
 
 		Returns:
-			string: A string value
+			string: A string representing the symbol
 		"""
 
 		return self.__symbol
@@ -80,7 +80,7 @@ class Currency(object):
 		The method to get the is_active
 
 		Returns:
-			bool: A bool value
+			bool: A bool representing the is_active
 		"""
 
 		return self.__is_active
@@ -104,7 +104,7 @@ class Currency(object):
 		The method to get the exchange_rate
 
 		Returns:
-			string: A string value
+			string: A string representing the exchange_rate
 		"""
 
 		return self.__exchange_rate
@@ -186,7 +186,7 @@ class Currency(object):
 		The method to get the prefix_symbol
 
 		Returns:
-			bool: A bool value
+			bool: A bool representing the prefix_symbol
 		"""
 
 		return self.__prefix_symbol
@@ -210,7 +210,7 @@ class Currency(object):
 		The method to get the is_base
 
 		Returns:
-			bool: A bool value
+			bool: A bool representing the is_base
 		"""
 
 		return self.__is_base
@@ -260,7 +260,7 @@ class Currency(object):
 		The method to get the name
 
 		Returns:
-			string: A string value
+			string: A string representing the name
 		"""
 
 		return self.__name
@@ -313,7 +313,7 @@ class Currency(object):
 		The method to get the id
 
 		Returns:
-			int: A int value
+			int: An int representing the id
 		"""
 
 		return self.__id
@@ -337,7 +337,7 @@ class Currency(object):
 		The method to get the iso_code
 
 		Returns:
-			string: A string value
+			string: A string representing the iso_code
 		"""
 
 		return self.__iso_code
@@ -364,7 +364,7 @@ class Currency(object):
 			key (string) : A string representing the key
 
 		Returns:
-			int: A int value
+			int: An int representing the modification
 		"""
 
 		if key is not None and not isinstance(key, str):
