@@ -20,7 +20,7 @@ class Properties(object):
 		The method to get the reorder_rows
 
 		Returns:
-			bool: A bool value
+			bool: A bool representing the reorder_rows
 		"""
 
 		return self.__reorder_rows
@@ -73,7 +73,7 @@ class Properties(object):
 		The method to get the maximum_rows
 
 		Returns:
-			int: A int value
+			int: An int representing the maximum_rows
 		"""
 
 		return self.__maximum_rows
@@ -100,7 +100,7 @@ class Properties(object):
 			key (string) : A string representing the key
 
 		Returns:
-			int: A int value
+			int: An int representing the modification
 		"""
 
 		if key is not None and not isinstance(key, str):
