@@ -46,7 +46,7 @@ class Criteria(object):
 		The method to get the field
 
 		Returns:
-			string: A string value
+			string: A string representing the field
 		"""
 
 		return self.__field
@@ -70,7 +70,7 @@ class Criteria(object):
 		The method to get the value
 
 		Returns:
-			Object: A Object value
+			Object: A Object representing the value
 		"""
 
 		return self.__value
@@ -142,7 +142,7 @@ class Criteria(object):
 			key (string) : A string representing the key
 
 		Returns:
-			int: A int value
+			int: An int representing the modification
 		"""
 
 		if key is not None and not isinstance(key, str):
