@@ -48,7 +48,7 @@ class FileBodyWrapper(ResponseHandler):
 			key (string) : A string representing the key
 
 		Returns:
-			int: A int value
+			int: An int representing the modification
 		"""
 
 		if key is not None and not isinstance(key, str):
