@@ -9,22 +9,24 @@ class APIResponse(object):
 
         """
         Creates an APIResponse class instance with the specified parameters.
-        :param headers: A dict containing the API response headers.
-        :param status_code: An integer containing the API response HTTP status code.
-        :param object: A object containing the API response POJO class instance.
+
+        Parameters:
+             headers (dict) : A dict containing the API response headers.
+             status_code (int) : An integer containing the API response HTTP status code.
+             object (object) : An object containing the API response class instance.
         """
 
         self.__headers = headers
-
         self.__status_code = status_code
-
         self.__object = object
 
     def get_headers(self):
 
         """
         This is a getter method to get API response headers.
-        :return: A dict representing the API response headers.
+
+        Returns:
+            dict: A dict representing the API response headers.
         """
 
         return self.__headers
@@ -33,7 +35,9 @@ class APIResponse(object):
 
         """
         This is a getter method to get the API response HTTP status code.
-        :return: An integer representing the API response HTTP status code.
+
+        Returns:
+            int: An integer representing the API response HTTP status code.
         """
 
         return self.__status_code
@@ -41,8 +45,10 @@ class APIResponse(object):
     def get_object(self):
 
         """
-        This method to get an API response POJO class instance.
-        :return: A class instance.
+        This method to get an API response class instance.
+
+        Returns:
+            object: An object containing the API response class instance.
         """
 
         return self.__object
