@@ -1,4 +1,11 @@
-import os
+try:
+    import os
+    from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
+    from zcrmsdk.src.com.zoho.crm.api.util import Constants
+except Exception:
+    import os
+    from ..exception import SDKException
+    from .constants import Constants
 
 
 class StreamWrapper(object):
@@ -19,6 +26,9 @@ class StreamWrapper(object):
         """
 
         if file_path is not None:
+            if not os.path.exists(file_path):
+                raise SDKException(Constants.FILE_ERROR, Constants.FILE_DOES_NOT_EXISTS)
+
             self.__name = os.path.basename(file_path)
             self.__stream = open(file_path, 'rb')
 
