@@ -8,8 +8,7 @@ try:
     import zlib
     import base64
     import re
-    from zcrmsdk.src.com.zoho.crm.api.util.constants import Constants
-    from zcrmsdk.src.com.zoho.crm.api.util.converter import Converter
+    from zcrmsdk.src.com.zoho.crm.api.util import Constants
     from zcrmsdk.src.com.zoho.crm.api.initializer import Initializer
     from zcrmsdk.src.com.zoho.crm.api.exception import SDKException
     from zcrmsdk.src.com.zoho.crm.api.header_map import HeaderMap
@@ -25,7 +24,6 @@ except Exception:
     import base64
     import re
     from .constants import Constants
-    from .converter import Converter
     from ..initializer import Initializer
     from ..exception import SDKException
     from ..header_map import HeaderMap
@@ -45,13 +43,38 @@ class Utility(object):
     logger = logging.getLogger('SDKLogger')
 
     @staticmethod
-    def get_fields(module_api_name):
+    def set_handler_api_path(module_api_name, handler_instance):
+
+        if handler_instance is None:
+            return
+
+        api_path = handler_instance.get_api_path()
+
+        if module_api_name.lower() in api_path.lower():
+            api_path_split = api_path.split("/")
+
+            for i in range(len(api_path_split)):
+                split_lower = api_path_split[i].lower()
+                if split_lower == module_api_name.lower():
+                    api_path_split[i] = module_api_name
+                elif Constants.DEFAULT_MODULENAME_VS_APINAME.get(split_lower) is not None:
+                    api_path_split[i] = Constants.DEFAULT_MODULENAME_VS_APINAME.get(split_lower)
+
+            api_path = "/".join(api_path_split)
+            handler_instance.set_api_path(api_path)
+
+
+
+
+    @staticmethod
+    def get_fields(module_api_name, handler_instance=None):
 
         """
         This method to fetch field details of the current module for the current user and store the result in a JSON file.
 
         Parameters:
             module_api_name (str) : A string containing the CRM module API name.
+            handler_instance (CommonAPIHandler) : A CommonAPIHandler instance.
         """
 
         try:
@@ -71,50 +94,28 @@ class Utility(object):
                         return
                     os.makedirs(resources_path)
 
+                module_api_name = Utility.verify_module_api_name(module_api_name)
+
+                Utility.set_handler_api_path(module_api_name, handler_instance)
+
                 record_field_details_path = Utility.get_file_name()
 
                 if os.path.exists(record_field_details_path):
-                    record_field_details_json = Initializer.get_json(record_field_details_path)
-
-                    if Initializer.get_initializer().sdk_config.get_auto_refresh_fields() and not Utility.new_file and not Utility.get_modified_modules and (Constants.FIELDS_LAST_MODIFIED_TIME not in record_field_details_json or Utility.force_refresh or (time.time() * 1000 - record_field_details_json[Constants.FIELDS_LAST_MODIFIED_TIME]) > 3600000):
-                        Utility.get_modified_modules = True
-                        last_modified_time = record_field_details_json[Constants.FIELDS_LAST_MODIFIED_TIME] if Constants.FIELDS_LAST_MODIFIED_TIME in record_field_details_json else None
-                        Utility.modify_fields(record_field_details_path, last_modified_time)
-                        Utility.get_modified_modules = False
-
-                    elif not Initializer.get_initializer().sdk_config.get_auto_refresh_fields() and Utility.force_refresh and not Utility.get_modified_modules:
-                        Utility.get_modified_modules = True
-                        Utility.modify_fields(record_field_details_path, last_modified_time)
-                        Utility.get_modified_modules = False
-
-                    record_field_details_json = Initializer.get_json(record_field_details_path)
-
-                    if module_api_name is None or module_api_name.lower() in record_field_details_json:
-                        return
-
-                    else:
-                        Utility.fill_data_type()
-                        record_field_details_json[module_api_name.lower()] = {}
-                        Utility.write_to_file(file_path=record_field_details_path, file_contents=record_field_details_json)
-                        field_details = Utility.get_fields_details(module_api_name)
-                        record_field_details_json = Initializer.get_json(record_field_details_path)
-                        record_field_details_json[module_api_name.lower()] = field_details
-                        Utility.write_to_file(file_path=record_field_details_path, file_contents=record_field_details_json)
-
+                    Utility.file_exists_flow(module_api_name, record_field_details_path, last_modified_time)
                 elif Initializer.get_initializer().sdk_config.get_auto_refresh_fields():
                     Utility.new_file = True
                     Utility.fill_data_type()
-                    module_api_names = Utility.get_modules(None)
-                    record_field_details_json = {
-                        Constants.FIELDS_LAST_MODIFIED_TIME: time.time() * 1000
-                    }
+                    api_supported_modules = Utility.get_modules(None)
+                    record_field_details_json = Initializer.get_json(record_field_details_path) if os.path.exists(record_field_details_path) else {}
 
-                    for module in module_api_names:
+                    record_field_details_json[Constants.FIELDS_LAST_MODIFIED_TIME] = time.time() * 1000
+
+                    for module in api_supported_modules:
                         if module.lower() not in record_field_details_json:
                             record_field_details_json[module.lower()] = {}
                             Utility.write_to_file(file_path=record_field_details_path,
                                                   file_contents=record_field_details_json)
-                            field_details = Utility.get_fields_details(module)
+                            field_details = Utility.get_fields_details(api_supported_modules[module][Constants.API_NAME])
                             record_field_details_json = Initializer.get_json(record_field_details_path)
                             record_field_details_json[module.lower()] = field_details
                             Utility.write_to_file(file_path=record_field_details_path,
@@ -176,14 +177,63 @@ class Utility(object):
         record_field_details_json[Constants.FIELDS_LAST_MODIFIED_TIME] = time.time() * 1000
         Utility.write_to_file(file_path=record_field_details_path, file_contents=record_field_details_json)
         if len(modified_modules) > 0:
-            for module in modified_modules:
+            for module in modified_modules.keys():
                 if module.lower() in record_field_details_json:
                     Utility.delete_fields(record_field_details_json, module)
 
             Utility.write_to_file(file_path=record_field_details_path, file_contents=record_field_details_json)
 
-            for module in modified_modules:
-                Utility.get_fields(module)
+            for module in modified_modules.keys():
+                module_data = modified_modules[module]
+                Utility.get_fields(module_data[Constants.API_NAME])
+
+    @staticmethod
+    def file_exists_flow(module_api_name, record_field_details_path, last_modified_time):
+        with Utility.lock:
+            record_field_details_json = Initializer.get_json(record_field_details_path)
+
+            if Initializer.get_initializer().sdk_config.get_auto_refresh_fields() and not Utility.new_file and not Utility.get_modified_modules and (
+                    Constants.FIELDS_LAST_MODIFIED_TIME not in record_field_details_json or Utility.force_refresh or (
+                    time.time() * 1000 - record_field_details_json[Constants.FIELDS_LAST_MODIFIED_TIME]) > 3600000):
+                Utility.get_modified_modules = True
+                last_modified_time = record_field_details_json[
+                    Constants.FIELDS_LAST_MODIFIED_TIME] if not Utility.force_refresh and Constants.FIELDS_LAST_MODIFIED_TIME in record_field_details_json else None
+                Utility.modify_fields(record_field_details_path, last_modified_time)
+                Utility.get_modified_modules = False
+
+            elif not Initializer.get_initializer().sdk_config.get_auto_refresh_fields() and Utility.force_refresh and not Utility.get_modified_modules:
+                Utility.get_modified_modules = True
+                Utility.modify_fields(record_field_details_path, last_modified_time)
+                Utility.get_modified_modules = False
+
+            record_field_details_json = Initializer.get_json(record_field_details_path)
+
+            if module_api_name is None or module_api_name.lower() in record_field_details_json:
+                return
+
+            else:
+                Utility.fill_data_type()
+                record_field_details_json[module_api_name.lower()] = {}
+                Utility.write_to_file(file_path=record_field_details_path, file_contents=record_field_details_json)
+                field_details = Utility.get_fields_details(module_api_name)
+                record_field_details_json = Initializer.get_json(record_field_details_path)
+                record_field_details_json[module_api_name.lower()] = field_details
+                Utility.write_to_file(file_path=record_field_details_path, file_contents=record_field_details_json)
+
+    @staticmethod
+    def verify_module_api_name(module_api_name):
+        if module_api_name is not None and Constants.DEFAULT_MODULENAME_VS_APINAME.get(module_api_name.lower()) is not None:
+            return Constants.DEFAULT_MODULENAME_VS_APINAME.get(module_api_name.lower())
+
+        record_field_details_path = Utility.get_file_name()
+        if not os.path.exists:
+            os.makedirs(record_field_details_path)
+
+        fields_json = Initializer.get_json(record_field_details_path)
+        if Constants.SDK_MODULE_METADATA in fields_json and module_api_name.lower() in fields_json[Constants.SDK_MODULE_METADATA]:
+            return fields_json[Constants.SDK_MODULE_METADATA][module_api_name.lower()][Constants.API_NAME]
+
+        return module_api_name
 
     @staticmethod
     def delete_fields(record_field_details_json, module):
@@ -201,8 +251,10 @@ class Utility(object):
 
     @staticmethod
     def get_file_name():
+        import zcrmsdk.src.com.zoho.crm.api.util.converter as Converter
+
         return os.path.join(Initializer.get_initializer().resource_path, Constants.FIELD_DETAILS_DIRECTORY,
-                            Converter.get_encoded_file_name())
+                            Converter.Converter.get_encoded_file_name())
 
     @staticmethod
     def get_related_lists(related_module_name, module_api_name, common_api_handler):
@@ -253,12 +305,68 @@ class Utility(object):
                     raise SDKException(code=Constants.UNSUPPORTED_IN_API, message=common_api_handler.get_http_method() + ' ' + common_api_handler.get_api_path() + Constants.UNSUPPORTED_IN_API_MESSAGE)
 
                 if related_list_jo[Constants.MODULE] != Constants.NULL_VALUE:
-                    common_api_handler.set_module_api_name(related_list_jo[Constants.MODULE])
+                    common_api_handler.set_module_api_name(related_list_jo[Constants.MODULE], common_api_handler)
                     Utility.get_fields(related_list_jo[Constants.MODULE])
                 return True
 
         return False
 
+    @staticmethod
+    def verify_photo_support(module_api_name):
+        with Utility.lock:
+            try:
+                module_api_name = Utility.verify_module_api_name(module_api_name)
+                if module_api_name in Constants.PHOTO_SUPPORTED_MODULES:
+                    return True
+
+                modules = Utility.get_module_names()
+                if module_api_name.lower() in modules and modules[module_api_name.lower()] is not None:
+                    module_meta_data = modules[module_api_name.lower()]
+
+                    if module_meta_data[Constants.GENERATED_TYPE] != Constants.GENERATED_TYPE_CUSTOM:
+                        raise SDKException(code = Constants.UPLOAD_PHOTO_UNSUPPORTED_ERROR, message= Constants.UPLOAD_PHOTO_UNSUPPORTED_MESSAGE + module_api_name)
+            except SDKException as e:
+                raise e
+
+            except Exception as e:
+                sdk_exception = SDKException(message=Constants.EXCEPTION, cause=e)
+                raise sdk_exception
+
+            return True
+
+    @staticmethod
+    def get_module_names():
+        module_data = {}
+        resources_path = os.path.join(Initializer.get_initializer().resource_path,
+                                      Constants.FIELD_DETAILS_DIRECTORY)
+        if not os.path.exists(resources_path):
+            os.makedirs(resources_path)
+
+        record_field_details_path = Utility.get_file_name()
+
+        is_null = False
+        if os.path.exists(record_field_details_path):
+            json = Initializer.get_json(record_field_details_path)
+            if Constants.SDK_MODULE_METADATA not in json:
+                is_null = True
+            elif json[Constants.SDK_MODULE_METADATA] is None:
+                is_null = True
+
+        if not os.path.exists(record_field_details_path) or is_null:
+            module_data = Utility.get_modules(None)
+            Utility.write_module_meta_data(record_field_details_path, module_data)
+            return module_data
+
+        record_field_details_json = Initializer.get_json(record_field_details_path)
+        module_data = record_field_details_json[Constants.SDK_MODULE_METADATA]
+        return module_data
+
+    @staticmethod
+    def write_module_meta_data(record_field_details_path, module_data):
+        field_details_json = Initializer.get_json(record_field_details_path) if os.path.exists(record_field_details_path) else {}
+        field_details_json[Constants.SDK_MODULE_METADATA] = module_data
+        Utility.write_to_file(record_field_details_path,field_details_json)
+
     @staticmethod
     def get_related_list_details(module_api_name):
         import zcrmsdk.src.com.zoho.crm.api.related_lists as RelatedLists
@@ -293,19 +401,19 @@ class Utility(object):
                         Constants.STATUS: data_object.get_status().get_value(),
                         Constants.MESSAGE: data_object.get_message().get_value()
                     }
-                    raise SDKException(Constants.API_EXCEPTION, None, error_response)
+                    raise SDKException(code=Constants.API_EXCEPTION, details=error_response)
 
                 else:
                     error_response = {
                         Constants.CODE: response.get_status_code()
                     }
-                    raise SDKException(Constants.API_EXCEPTION, None, error_response)
+                    raise SDKException(code=Constants.API_EXCEPTION, details=error_response)
 
             else:
                 error_response = {
                     Constants.CODE: response.get_status_code()
                 }
-                raise SDKException(Constants.API_EXCEPTION, None, error_response)
+                raise SDKException(code=Constants.API_EXCEPTION, details=error_response)
 
         return related_list_array
 
@@ -341,7 +449,7 @@ class Utility(object):
 
                     fields_details[field.get_api_name()] = field_detail
 
-                if module_api_name in Constants.INVENTORY_MODULES:
+                if module_api_name.lower() in Constants.INVENTORY_MODULES:
                     field_detail = {
                         Constants.NAME: Constants.LINE_TAX,
                         Constants.TYPE: Constants.LIST_NAMESPACE,
@@ -363,13 +471,13 @@ class Utility(object):
                     Constants.STATUS: response_object.get_status().get_value(),
                     Constants.MESSAGE: response_object.get_message().get_value()
                 }
-                raise SDKException(Constants.API_EXCEPTION, None, error_response)
+                raise SDKException(code=Constants.API_EXCEPTION, details=error_response)
 
         else:
             error_response = {
                 Constants.CODE: response.get_status_code()
             }
-            raise SDKException(Constants.API_EXCEPTION, None, error_response)
+            raise SDKException(code=Constants.API_EXCEPTION, details=error_response)
         return fields_details
 
     @staticmethod
@@ -394,7 +502,7 @@ class Utility(object):
 
         import zcrmsdk.src.com.zoho.crm.api.modules as Modules
 
-        api_names = []
+        api_names = {}
         header_map = HeaderMap()
         if header is not None:
             header_value = datetime.datetime.fromtimestamp(header / 1000.0)
@@ -412,7 +520,10 @@ class Utility(object):
                 modules = response_object.get_modules()
                 for module in modules:
                     if module.get_api_supported():
-                        api_names.append(module.get_api_name())
+                        module_details = dict()
+                        module_details[Constants.API_NAME] = module.get_api_name()
+                        module_details[Constants.GENERATED_TYPE] = module.get_generated_type().get_value()
+                        api_names[module.get_api_name().lower()] = module_details
 
             elif isinstance(response_object, Modules.APIException):
                 error_response = {
@@ -420,13 +531,14 @@ class Utility(object):
                     Constants.STATUS: response_object.get_status().get_value(),
                     Constants.MESSAGE: response_object.get_message().get_value()
                 }
-                raise SDKException(Constants.API_EXCEPTION, None, error_response)
+                raise SDKException(code=Constants.API_EXCEPTION, details=error_response)
 
-        else:
-            error_response = {
-                Constants.CODE: response.get_status_code()
-            }
-            raise SDKException(Constants.API_EXCEPTION, None, error_response)
+        if header is None:
+            try:
+                Utility.write_module_meta_data(Utility.get_file_name(), api_names)
+            except Exception as e:
+
+                raise SDKException(message=Constants.EXCEPTION, cause=e)
 
         return api_names
 
@@ -444,6 +556,20 @@ class Utility(object):
 
         return None
 
+    @staticmethod
+    def check_data_type(value, type):
+
+        if value is None:
+            return False
+        if type.lower() == Constants.OBJECT.lower():
+            return True
+        type = Constants.DATA_TYPE.get(type)
+        class_name = value.__class__
+        if class_name == type:
+            return True
+        else:
+            return False
+
     @staticmethod
     def set_data_type(field_detail, field, module_api_name):
         api_type = field.get_data_type()
@@ -453,7 +579,7 @@ class Utility(object):
         if field.get_system_mandatory() is not None and field.get_system_mandatory() and not(module_api_name.lower() == Constants.CALLS and key_name.lower() == Constants.CALL_DURATION):
             field_detail[Constants.REQUIRED] = True
 
-        if key_name.lower() == Constants.PRODUCT_DETAILS.lower() and module_api_name.lower() in Constants.INVENTORY_MODULES:
+        if  key_name.lower() in Constants.KEY_VS_INVENTORY_MODULE and module_api_name.lower() == Constants.KEY_VS_INVENTORY_MODULE.get(key_name.lower()):
             field_detail[Constants.NAME] = key_name
             field_detail[Constants.TYPE] = Constants.LIST_NAMESPACE
             field_detail[Constants.STRUCTURE_NAME] = Constants.INVENTORY_LINE_ITEMS
@@ -494,6 +620,13 @@ class Utility(object):
 
             return
 
+        elif key_name.lower() == Constants.TERRITORIES.lower() and field.get_custom_field() is not None and not field.get_custom_field():
+            field_detail[Constants.NAME] = key_name
+            field_detail[Constants.TYPE] = Constants.LIST_NAMESPACE
+            field_detail[Constants.STRUCTURE_NAME] = Constants.TERRITORY_NAMESPACE
+            field_detail[Constants.LOOKUP] = True
+
+            return
         elif api_type in Utility.apitype_vs_datatype:
             field_detail[Constants.TYPE] = Utility.apitype_vs_datatype.get(api_type)
 
@@ -508,15 +641,36 @@ class Utility(object):
         else:
             return
 
-        if Constants.LOOKUP in api_type.lower():
+        if api_type.lower() in Constants.LOOKUP:
             field_detail[Constants.LOOKUP] = True
 
-        if api_type.lower() == Constants.CONSENT_LOOKUP:
+        if api_type.lower() in Constants.CONSENT_LOOKUP:
             field_detail[Constants.SKIP_MANDATORY] = True
 
+        if api_type.lower() in Constants.MULTI_SELECT_LOOKUP:
+            field_detail[Constants.SKIP_MANDATORY] = True
+            if field.get_multiselectlookup() is not None:
+                if field.get_multiselectlookup().get_linking_module() is not None:
+                    linking_module = field.get_multiselectlookup().get_linking_module()
+                    field_detail[Constants.MODULE] = linking_module
+                    module = linking_module
+
+            field_detail[Constants.SUBFORM] = True
+
+        if api_type.lower() in Constants.MULTI_USER_LOOKUP:
+            field_detail[Constants.SKIP_MANDATORY] = True
+            if field.get_multiuserlookup() is not None:
+                if field.get_multiuserlookup().get_linking_module() is not None:
+                    linking_module = field.get_multiuserlookup().get_linking_module()
+                    field_detail[Constants.MODULE] = linking_module
+                    module = linking_module
+
+            field_detail[Constants.SUBFORM] = True
+
         if api_type in Utility.apitype_vs_structurename:
             field_detail[Constants.STRUCTURE_NAME] = Utility.apitype_vs_structurename.get(api_type)
 
+
         if api_type.lower() == Constants.PICKLIST and field.get_pick_list_values() is not None and len(field.get_pick_list_values()) > 0:
             field_detail[Constants.PICKLIST] = True
             values = list(map(lambda x: x.get_display_value(), field.get_pick_list_values()))
@@ -552,7 +706,7 @@ class Utility(object):
             return
 
         field_api_names_string = ["textarea", "text", "website", "email", "phone", "mediumtext", "multiselectlookup",
-                                  "profileimage"]
+                                  "profileimage", "autonumber"]
 
         field_api_names_integer = ['integer']
 
@@ -560,9 +714,7 @@ class Utility(object):
 
         field_api_names_double = ['double', 'percent', 'lookup', 'currency']
 
-        field_api_names_long = ['long', 'bigint', 'autonumber']
-
-        field_api_names_file = ['imageupload']
+        field_api_names_long = ['long', 'bigint']
 
         field_api_names_field_file = ['fileupload']
 
@@ -592,6 +744,10 @@ class Utility(object):
 
         field_api_name_consent_lookup = ['consent_lookup']
 
+        field_api_name_image_upload = ["imageupload"]
+
+        field_api_name_multiselect_lookup = ["multiselectlookup"]
+
         for field_api_name in field_api_names_string:
             Utility.apitype_vs_datatype[field_api_name] = Constants.STRING_NAMESPACE
 
@@ -607,9 +763,6 @@ class Utility(object):
         for field_api_name in field_api_names_long:
             Utility.apitype_vs_datatype[field_api_name] = Constants.LONG_NAMESPACE
 
-        for field_api_name in field_api_names_file:
-            Utility.apitype_vs_datatype[field_api_name] = Constants.FILE_NAMESPACE
-
         for field_api_name in field_api_names_datetime:
             Utility.apitype_vs_datatype[field_api_name] = Constants.DATETIME_NAMESPACE
 
@@ -626,7 +779,7 @@ class Utility(object):
 
         for field_api_name in field_api_names_multiuser_lookup:
             Utility.apitype_vs_datatype[field_api_name] = Constants.LIST_NAMESPACE
-            Utility.apitype_vs_structurename[field_api_name] = Constants.USER_NAMESPACE
+            Utility.apitype_vs_structurename[field_api_name] = Constants.RECORD_NAMESPACE
 
         for field_api_name in field_api_names_multimodule_lookup:
             Utility.apitype_vs_datatype[field_api_name] = Constants.LIST_NAMESPACE
@@ -663,6 +816,14 @@ class Utility(object):
             Utility.apitype_vs_datatype[field_api_name] = Constants.CONSENT_NAMESPACE
             Utility.apitype_vs_structurename[field_api_name] = Constants.CONSENT_NAMESPACE
 
+        for field_api_name in field_api_name_image_upload:
+            Utility.apitype_vs_datatype[field_api_name] = Constants.LIST_NAMESPACE
+            Utility.apitype_vs_structurename[field_api_name] = Constants.IMAGEUPLOAD_NAMESPACE
+
+        for field_api_name in field_api_name_multiselect_lookup:
+            Utility.apitype_vs_datatype[field_api_name] = Constants.LIST_NAMESPACE
+            Utility.apitype_vs_structurename[field_api_name] = Constants.RECORD_NAMESPACE
+
     @staticmethod
     def write_to_file(file_path, file_contents):
         with open(file_path, mode="w") as file:
